<?php

class Everyboard_Dashboard
{

    private $versions_url = 'https://composer.infomaker.io/versions.json';

    public function __construct()
    {
        add_action('wp_dashboard_setup', [&$this, 'eb_add_dashboard_widget']);
    }

    /**
     * Add a widget to the admin dashboard.
     */
    function eb_add_dashboard_widget()
    {
        wp_add_dashboard_widget(
            'eb_dashboard_widget',
            'Everyboard',
            [&$this, 'eb_dashboard_widget_render']
        );
    }

    /**
     * Outputs data to dashboard widget.
     */
    function eb_dashboard_widget_render()
    {
        $latest_version = $this->get_latest_version();

        print '<p>';
        print __('Currently running Everyboard version: ',
                'everyboard') . '<strong>' . EVERYBOARD_VERSION . '</strong>';
        print '</p>';

        if ( empty($latest_version) ) {
            print '<p>';
            print __('Could not fetch version information, check back later!', 'everyboard');
            print '</p>';
        } elseif ($latest_version === EVERYBOARD_VERSION) {
            print '<p>';
            print __('Up to date with latest release.', 'everyboard');
            print '</p>';
        } elseif (version_compare(EVERYBOARD_VERSION, $latest_version, '<')) {
            print '<p>';
            print __('There is a newer version of Everyboard available, update or contact Infomaker for more information.',
                    'everyboard') . ' ';
            print '<strong>' . __('Latest version', 'everyboard') . ': ' . $latest_version . '</strong>';
            print '</p>';
        }
        print '<hr>';
        print '<p style="color: #ddd;">';
        print '<a href="https://docs.infomaker.io/everyware/plugins/everyboard/" target="_blank">Documentation <span class="screen-reader-text">(opens in a new window)</span><span aria-hidden="true" class="dashicons dashicons-external"></span></a>';
        print ' | ';
        print '<a href="https://docs.infomaker.io/everyware/plugins/everyboard/changelog" target="_blank">Changelog <span class="screen-reader-text">(opens in a new window)</span><span aria-hidden="true" class="dashicons dashicons-external"></span></a>';
        print '</p>';
    }

    private function get_latest_version()
    {
        $key = 'eb_latest_version';

        if (false !== ($latest_version = get_transient($key))) {
            return $latest_version;
        }

        $versions = $this->get_available_versions();

        if (empty($versions)) {
            return '';
        }

        usort($versions, 'version_compare');

        $latest_version = array_pop($versions);

        set_transient($key, $latest_version, HOUR_IN_SECONDS);

        return $latest_version;
    }

    private function get_available_versions()
    {
        $response = wp_remote_get($this->versions_url, ['timeout' => 3]);

        // Check the response code
        $response_code = (int)wp_remote_retrieve_response_code($response);
        $response_message = wp_remote_retrieve_response_message($response);

        if ($response_code !== 200) {
            return new WP_Error($response_code,
                ! empty($response_message) ? $response_message : 'Could not fetch latest version');
        }

        $plugins = json_decode(wp_remote_retrieve_body($response), false);

        $versions = [];
        foreach ($plugins as $plugin) {
            if ($plugin->name === 'infomaker/everyboard') {
                $versions = $plugin->versions;
                break;
            }
        }

        return $versions;
    }
}
